@@ -13,7 +13,7 @@
     if (missing(leftout)) n <- length(lp) else n <- sum(!leftout)
     loglik <- (-n/2) * (log(2*pi/n) + 1 + log(ss + .Machine$double.xmin))
 
-    return(list(residuals = residuals, loglik = loglik, W = 1, sigma2 = ss/n, lp = lp))
+    return(list(residuals = residuals, loglik = loglik, W = 1, lp = lp, fitted = lp, nuisance = list(sigma2 = ss/n), lp = lp))
   }
 
   cvl <- function(lp, leftout) {
@@ -24,8 +24,15 @@
 
     return(-(sum(leftout)/2) * log(2*pi*sigma2) - ss / (2*sigma2))
   }
+  
+  # mapping from the linear predictor lp to an actual prediction
+  prediction <- function(lp, nuisance) {
+    out <- c(mu = lp, sigma2 = nuisance$sigma2)
+    out
+  }
 
-  return(list(fit = fit, cvl = cvl))
+
+  return(list(fit = fit, cvl = cvl, prediction = prediction))
 }
 
 
@@ -43,4 +50,12 @@
     form <- as.formula(paste(".response~", paste(terms, collapse="+")))
     startgamma <- coefficients(lm(form, data = data))
   }
-}
\ No newline at end of file
+}
+
+# merges predicted means and variances
+.lmmerge <- function(predictions) {
+  out <- matrix(unlist(predictions), length(predictions), 2, byrow=TRUE)
+  colnames(out) <- c("mu", "sigma2")
+  rownames(out) <- names(predictions)
+  out
+}
